      PROGRAM NRATTLE

!  Haskell-Thomson transfer function (SH) for horizontally stratified medium.
!  Calculate complex spectrum from DC to Nyquist.

! The transfer function is normalized by twice the amplitude of the incoming
! wave so that the amplitude of the transfer function approaches
! unity for low frequencies.   This is equivalent to the transfer function
! being the ratio of site response at the surface above the soil layer and
! the site response at the surface of a "rock" site that is a uniform material
! with velocity and density equal to those in the halfspace below the
! layers.  If the complex transfer function is to be used to
! deconvolve surface motions to obtain the input motion below the layers, the 
! transfer function should first be multiplied by 2.0 before dividing the 
! spectrum of the surface motion by the complex transfer function.  If that 
! input motion is then to be convolved with the transfer function for a 
! different site, the factor of two has to be accounted for again.  The result
! is that the factors of two cancel, so that the process of deconvolution and
! convolution is given simply by multiplying the spectrum of the observed
! motion at site 1 (S_1) by the ratio of the complex transfer functions
! at sites 1 and 2: S_2 = S_1 * (T_2/T_1)

!
!  Haskell's approach: Propagate particle velocity and stress through
!    (m)th layer [from (m-1)st boundary to (m)th boundary].
!  This approach: Propagate coefficients of up- and downgoing waves
!    across (m)th boundary [from (m+1)st layer to (m)th layer].
!  Mixed units are OK. Everything cancels.
!
!  C. Mueller redid this from scratch - not like Joyner's anymore. (7/84, CSM).
!  FFT sign conventions are Aki-Richards (Box 5.2).


! Sample input (from file nrattle.ctl) (the line below is line 1 of the input):
!!Control file for nrattle (data lines are free format)
!! (note that text lines preceded by "!" are ignored)
!!Revision of program involving a change in the control file on this date:
!    09/09/11
!!Stem name for output files (the file containing model information and complex spectra will be named
!! <stem>.nrattle_complex_spectra.out
!! and the one containing the absolute spectral amplitudes used in plotting will be named
!! <stem>.nrattle_amps4plot.out
!  dmb_interpolate_bj97_gr_vhr_to_760mps.mdl.site_amp_batch.1200_freqs.test
!!Number of frequencies (including f = 0.0) and highest freq. (nfreq, fhigh):
!! If nfreq and fhigh are being specified for use in creating a time series via an FFT,
!!         nfreq = npts/2 + 1
!!         fhigh = 1.0/(2.0*dt) (the Nyquist frequency)
!! where npts is the number of points in the FFT (a power of 2)
!  1200   100.00
!!Depth at which response is computed (0.0 for free surface):
!    0.00
!!Model information:
!!  Nlayers above the actual halfspace; fmin4model:
!! 1200 9.539E-02
!!For each layer- thickness, Vs, density, 1/Q, depth-to-bottom of current layer
!! NOTE: depth-to-bottom is computed and written to nrattle.ctl by f4nrattle; depth-to-bottom is not read by nrattle
!! (if 1/Q > 1.0, the program assumes that the value represents Q; note that damping in percent = 100*0.5*1/Q):
!    1 7.84239E-04 3.13695E-01 1.93369E+00 0.00000E+00 7.84239E-04
!    2 6.04760E-06 3.13695E-01 1.93369E+00 0.00000E+00 7.90287E-04
!    3 6.09382E-06 3.13694E-01 1.93369E+00 0.00000E+00 7.96380E-04
!    4 6.14132E-06 3.13699E-01 1.93369E+00 0.00000E+00 8.02522E-04
!    5 6.18864E-06 3.13695E-01 1.93369E+00 0.00000E+00 8.08710E-04
!    .           .           .           .           .           .
!    .           .           .           .           .           .
!    .           .           .           .           .           .
! 1199 6.64452E-01 3.50002E+00 2.72003E+00 0.00000E+00 8.56574E+01
! 1200 6.69563E-01 3.49997E+00 2.71999E+00 0.00000E+00 8.63270E+01
!!  Halfspace Vs and density (1/Q is automatically set to 0.0):
!    3.50  2.72
!!Layer corresponding to halfspace (can be less than actual halfspace)
!! (a large number means the source is in the halfspace) and
!! angle of incidence (< 0.0 means obtain theta from site_amp file):
!  1201   0.00

! The line above is the last line of the input.

 
! If the layer is to be in the halfspace, then the layer number corresponding
! to the halfspace can be set to be a large integer; the program replaces it 
! with the proper value.  The only critical integer is the number of layers 
! above the real halfspace.

! Note that the text lines between the data lines are required (not the 
! text itself, only a line, which can be blank), with 2 or 3 lines required in 
! several places

! Site response will be computed for evenly spaced frequencies between 0.0 and
! the maximum frequency.

! The program as written by C. Mueller specified input parameters appropriate
! for using the response in a separate program that multiplied the complex
! response by the FFT of the input time series to obtain an output time
! series.  This required specifying nfreq and fhigh compatible with
! the FFT.  He found it most convenient to specify parameters SAMP and MX for
! this.  These are related to nfreq and fhigh as follows:

!         mx  = 1 + ln(nfreq -1)/ln(2)
!         samp = 2*fhigh
!

! If nfreq and fhigh are being specified for use in creating a time series via an FFT, 
!         nfreq = npts/2 + 1
!         fhigh = 1.0/(2.0*dt) (the Nyquist frequency)
! where npts is the number of points in the FFT (a power of 2) 

! The original version contained the following code to convert the
! input parameters MX and SAMP into the number of frequency points (INYQ, where
! this is the number of evenly spaced samples between f = 0.0 and the Nyquist
! frequency), the sample interval (DT), the length of the FFT time series (T),
! and the frequency spacing (DELF).

!      DT = 1.0/SAMP
!      IPW2 = 2**MX !Number of points in corresponding time series.
!      INYQ = IPW2/2+1
!      T = IPW2/SAMP
!      DELF = 1.0/T

!
!  Written by Chuck Mueller, USGS.
!    Changed input to skip comment lines at top of input file, 8/86.
! Unless indicated, all mods below by D.M. Boore
! Dates: 12/18/95 - Dave Boore modified the program for the pc and increased
!                   array dimensions, used list-directed input
!        03/12/96 - i/o changes, asking for file names and writing a file
!                   with fewer header lines for use in COPLOT (to be imported
!                   as a wrapped ASCII file).  Also added file stem name
!                   to output column.
!        01/28/98 - Read nlay_src from the last line of the input.  Use this
!                   to determine of the effective halfspace should be above 
!                   the actual halfspace of the model.  In this way, can run
!                   rattle with various source depths without needing to
!                   manually edit the input file.  Note that if the source
!                   layer is to the real halfspace, then set 
!                   nlay_src = nlay + 1
!        02/24/98 - Suggest rattle.asc for file to be imported into CoPlot, 
!                   rather than rattle.4cp.  Also, include relation between
!                   sps, mx and fhigh, nfreq (f4rattle now asks for the more
!                   natural fhigh, nfreq, and computes sps and mx.  Rattle
!                   still expects the latter as input).
!-----
!        07/27/98 - Reformulated by R. Herrmann to
!                   a) use a stabler formulation for surface motion that
!                      is less susceptible to numerical instabilities for
!                      low Q media
!                   b) Did not use Aki + Richards definition of 
!                      Fourier Transform
!                      Instead
!                      G(f) = int from { - inf } to { inf } 
!                            g(t) exp ( -j 2 pi f t ) dt
!                      g(t) = int from { - inf } to { inf } 
!                            G(f) exp (  j 2 pi f t ) df
!                      The controls the definition of upward and downward
!                      wavefields and the sign for the affect of 
!                      Q on complex velocity
!-----
!        10/31/98 - Read in 1/Q rather than Q (convert input to 1/Q
!                   if the input value is greater than 1.0)
!        11/05/98 - Suggest *.out and *.asc output file names
!        01/11/99 - Complete revamping of input parameters (see text above for
!                   new format and parameters)
!        10/11/00 - Corrected error in sample control file above
!        12/22/00 - Clarified choice of nfreq, fhigh for use in FFT
!        04/16/01 - Allow for variable number of comment lines in control file
!                   by using subroutine skipcmnt.  If there is more than one
!                   comment lines, then each line must be preceded by "|".
!        04/16/01 - Bring in subroutines via include nrtlsubs.for, where
!                   nrtlsubs.for is made each time the program is compiled
!                   by calling clnrattl.bat.  This guarantees that the latest
!                   versions of the subroutines are used.
!        05/01/01 - If effective half space is less than the actual half space,
!                   the old version still used the velocity and density given
!                   in the last line of the control file.  I (DMB) has changed
!                   this so that betan and rhon are given by the values in the
!                   layer that is the effective half space. 
!        02/10/03 - Use trim_c to strip leading and trailing blanks from
!                   file names (modern compilers allow blanks in file names,
!                   and this might produce errors if the file name obtained 
!                   from a control file is used as is--- for example, if
!                   the name in the control file is preceeded by blanks).
!        02/10/03 - Commented out computation of sumt, sumtq; those 
!                   quantities are not part of the nrattle standard output.
!        08/19/05 - Print name of output file to screen.
!        01/24/08 - Modernize by using get_lun, get subroutines from nrtlsubs.for
!                   (this is created by clnrttl.bat)
!        04/06/10 - Added period to output for plotting.
!        05/12/10 - Add depth to bottom of each layer to output file containing
!                   the model (nu_out).
!                 - Call system "date_and_time" routine rather than the compiler-dependent
!                   "datetime.for"
!        05/14/10 - Remove units of velocity, thicknesses from output files
!                   (the units can be mks or cgs or anything else as long as they
!                   are consistent).
!        10/17/10 - Allow for longer output filenames
!        09/09/11 - Add a change date to the control file, and read the stem name for
!                   the two output files rather than the names of the two output
!                   files.
!        09/11/11 - Add name of output file to output
!        09/13/11 - Delete specification of number of model layers in control file,
!                   and put the layer number for the effective halfspace at the bottom
!                   of the control file.
!        03/02/12 - Increase precision of frequency output, and write as fixed format.
!        08/18/12 - Allow more layers in model
!        08/20/12 - Allow more layers in model
!        03/08/13 - Increased dimension of comment
!        12/20/15 - Write nlayrs before model, so know how many lines to skip to get to the
!                   output when reading the file in another program.
!                 - Change spacing of some of the output lines.
!        11/29/21 - Added new sample nrattle.ctl lines above, to make it consistent with the current version
!                   of f4nrattle output
!        02/02/24 - Put the amp computations into a subroutine
!        02/06/24 - Moved computation of frequency out of the subroutine NrattleAmpComputations. This was done
!                   to make the frequencies more general--they could be log spaced, and could
!                   start at flow rather than 0.0.  The previous version of the subroutine
!                   implicitly assumed frequencies that were consistent with a fft, where
!                   fhigh was the nyquist frequency and the sample rate was 2*fhigh.
!                   The relation between fhigh, nfreq (and flow) is given in above.
!                   Note that because I do not want to revised the control file, and I want to make a minimal
!                   change to the program, I am hardwiring flow = 0.0 in this version and not allowing for log-spaced frequencies.  

      parameter(MLAYR=10000, MNFREQS=4000)
      real Z(MLAYR), H(MLAYR), vs(MLAYR), rho(mlayr), qis(MLAYR)
      real freq(MNFREQS)
      complex CmplxAmp(MNFREQS)
      character date_c*8, time_c*10
      character stem_name*200, f_ctl*300, f_out*300, f_4plt*300
      character tempbuf*8, col1*12, col2*12, col3*12
      character comment(200)*100, buf*200

      character date_ctl_correct*8, date_ctl_in*30
      
      logical f_ctl_exist

!**********
!
!  GET INPUT
!
!**********

      iflag = 0
      call date_and_time(date_c,time_c)
 
!...Open the control file:

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a)') 
     :    ' Enter name of control file ("Enter"=nrattle.ctl; '//
     :    'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'nrattle.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      
      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

!Check version of control file
      date_ctl_correct = ' '
      date_ctl_correct = '09/09/11'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      call skipcmnt(nu_ctl, comment, ncomments)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      if (date_ctl_correct(1:nc_date_ctl_correct) /=
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; STOP!'
        close(nu_ctl)
        stop
      end if
      
! Get stem name for output files, construct the output file names, and open the files:

      call skipcmnt(nu_ctl, comment, ncomments)
      stem_name = ' '
      read(nu_ctl, '(a)') stem_name
      call trim_c(stem_name,nc_stem_name)

      f_out = ' '
      f_out = stem_name(1:nc_stem_name)//
     :        '.complex_spectra.out'
      call trim_c(f_out,nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out, file=f_out(1:nc_f_out), status='unknown')
      
      f_4plt = ' '
      f_4plt = stem_name(1:nc_stem_name)//
     :        '.amps4plot.out'
      call trim_c(f_4plt, nc_f_4plt)
      call get_lun(nu_4plt)
      open(unit=nu_4plt, file=f_4plt(1:nc_f_4plt), status='unknown')

! Get nfreq and fhigh and convert to quantities used in calculations

      call skipcmnt(nu_ctl, comment, ncomments)

      read(nu_ctl,*) nfreq, fhigh

! Get depth at which response is computed:

      call skipcmnt(nu_ctl, comment, ncomments)

      read (nu_ctl,*) zdepth

! Get model:

      call skipcmnt(nu_ctl, comment, ncomments)

      zcum = 0.0
      i = 0
      
      buf = ' '
      read (nu_ctl, '(a)') buf
      call trim_c(buf,nc_buf)
      
      do while (buf(1:1) /= '!')
      
        i = i + 1
!debug
         print *, ' buf: '//buf(1:nc_buf)
         print *, ' i = ', i
!debug
        read (buf(1:nc_buf),*) ilay, h(i),vs(i),rho(i), qis(i)
        zcum = zcum + h(i)
        z(i) = zcum
        if (qis(i) .gt. 1.0) qis(i) = 1.0/qis(i)
        buf = ' '
        read (nu_ctl, '(a)') buf
        call trim_c(buf,nc_buf)
 
      END DO
      
! Get number of layers:

      nlay = i
      
      backspace nu_ctl  ! need this in order to skip next set of header lines

! Get number of layers:

! Get halfspace material properties:

      call skipcmnt(nu_ctl, comment, ncomments)
      
      read (nu_ctl,*) betan, rhon

!-----RBH
!	n is the working halfspace
!	Depending on the input, we may truncate the base of the model
!	The halfspace is assumed perfectly elastic
!-----RBH

! Get layer corresponding to halfspace and angle of incidence

      call skipcmnt(nu_ctl, comment, ncomments)

      read(nu_ctl,*) nlay_src, theta
!**********
!
!  DO COMPUTATIONS
!
!**********

      delf = fhigh/real(nfreq - 1) 
      do i = 1, nfreq
        freq(i) = real(i - 1) * delf
      end do

      CALL NrattleAmpComputations(z, h, vs, rho, qis, 
     :        nlay, nlay_src, betan, rhon, theta, zdepth, 
     :        freq, nfreq, CmplxAmp, mlayr, mnfreqs )


!**********
!
!  WRITE OUTPUT
!
!**********

      write(nu_out, '(a)') ' Program NRATTLE'
      write(nu_out, '(a)') ' Output file: '//f_out(1:nc_f_out)
      write(nu_out, '(a)') 
     : ' Date: '//date_c(1:4)//'-'//date_c(5:6)//'-'//date_c(7:8)
      write(nu_out, '(a)') 
     : ' Time: '//time_c(1:2)//'h'//time_c(3:4)//'m'//time_c(5:10)//'s'
      write(nu_out,*)
      
      write(nu_4plt, '(a)') ' Program NRATTLE'
      write(nu_4plt, '(a)') ' Output file: '//f_4plt(1:nc_f_4plt)
      write(nu_4plt, '(a)')  
     :  ' Date: '//date_c(1:4)//'-'//date_c(5:6)//'-'//date_c(7:8)
      write(nu_4plt, '(a)')  
     : ' Time: '//time_c(1:2)//'h'//time_c(3:4)//'m'//time_c(5:10)//'s'
      write(nu_4plt,*)
      
      WRITE(nu_4plt,*)
      
      write(nu_out,90120) nfreq, fhigh, delf
90120 FORMAT (' nfreq = ',i6/
     :        ' fhigh (Hz) = ',f8.3/
     :        ' delta f (Hz) = ',f10.7)
      write(nu_out,*)

      write(nu_out,'(a,f10.4)')
     :  ' Depth at which response is computed = ', zdepth
      write(nu_out,*)
      write(nu_out,'(a,i4)') 
     :    ' total number of layers in model = ', nlay
      write(nu_out,*)

! Write column headers for model:

      write(nu_out,'(
     :   1x,a, 7x,a, 3x,a,
     :   10x,a, 1x,a, 7x,a
     :                                           )' )
     :  'lyr_num', 'd2bot', 'thickness', 
     :   'vs', 'density', '1/q' 

      do i = 1, nlay
      
         write(nu_out,'(
     :                      4x,i4, 1x,es11.4, 1x,es11.4, 
     :                      1x,es11.4, 1x,f7.4, 1x,f9.4
     :                                                 )' ) 
     :          i, z(i), h(i),
     :          vs(i), rho(i), qis(i)
         
      end do
      
      write(nu_out,*)
      write(nu_out,'(a,i4)') 
     :  ' Layer number of effective halfspace ='  ,nlay_src
      write(nu_out,'(a,f7.2)') 
     :  ' Angle of incidence in halfspace ='  ,theta
      write(nu_out,*)

      write(nu_4plt,'(a,i4)') 
     :  ' Layer number of effective halfspace ='  ,nlay_src
      write(nu_4plt,'(a,f7.2)') 
     :  ' Angle of incidence in halfspace ='  ,theta
      write(nu_4plt,*)

      write (nu_out,90123) betan, rhon
90123 FORMAT (' Substratum S-velocity =',F10.4/
     *        ' Substratum density =',F10.4)
      write(nu_out,*)

      WRITE(nu_out,90320)
90320 FORMAT (t4,'nfreq', t14,'real(s)', t26,'imag(s)',
     :        t41,'freq', t50,'cabs(s)' )

      write(nu_4plt,'(a,i4)') 
     :    ' total number of layers in model = ', nlay
      write(nu_4plt,*)

      write(nu_4plt,90123) betan, rhon

      write(nu_4plt,*)
      write(nu_4plt,90120) nfreq, fhigh, delf
      write(nu_4plt,*)

      tempbuf = ' '

      tempbuf = f_4plt(1:index(f_4plt,'.')-1)
      call trim_c(tempbuf,nchar_tb)
      col1 = ' '
      col1(9-nchar_tb:8) = tempbuf(1:nchar_tb)
      col1(9:12) = '-frq'
      col2 = ' '
      col2(9-nchar_tb:8) = tempbuf(1:nchar_tb)
      col2(9:12) = '-per'
      col3 = ' '
      col3(9-nchar_tb:8) = tempbuf(1:nchar_tb)
      col3(9:12) = '-amp'
      write(nu_4plt, '(1x,a, 1x,a, 1x,a)') col1, col2, col3
 

      loop over frequencies: DO IFRQ=1,NFREQ

          f = freq(ifrq)

         WRITE (nu_out,92300) IFRQ,CmplxAmp(ifrq), f, 
     :                        cabs(CmplxAmp(ifrq))
92300    FORMAT (t4,i5, 4(1x,1pe11.4))
         if (f == 0.0) then
           per = 1.0e+06
         else
           per = 1.0/f
         end if
         WRITE (nu_4plt,'(1x,f12.7, 2(1x,es12.4))') f, per, 
     :          cabs(CmplxAmp(ifrq))
         
       END DO loop over frequencies        


!	write(nu_db,*)'KAPPA=',tstar

      write(*,'(1x,a)') 
     :  ' Finished: output file for plot = '//f_4plt(1:nc_f_4plt)
      write(*,'(1x,a)') 
     :  ' Finished: output file with model and complex spectrum = '//
     :   f_out(1:nc_f_out)

      CLOSE (UNIT=nu_ctl)
      CLOSE (UNIT=nu_out)
      CLOSE (UNIT=nu_4plt)

!      close(unit=nu_db)

      STOP
      END

      include 'site_amp_util_subs.for'

!      include 'nrtlsubs.for'

     