      Program Site_Amp_Batch

! Convert a velocity and density model into site amplifications,
! using the square-root of the impedance.

! This is a batch version of site_amp; it has almost the same functionality,
! except that tabulate frequncies are not included as an option.

! The amplification is relative to the surface motion that would
! exist if the material were replaced with uniform material whose velocity 
! and density equals those at the source; for this reason, the amplification
! approaches unity for low frequencies if the source velocity and density 
! equal those in the halfspace of the input model.

! The velocity model can be a combination of constant velocity layers and
! linear gradients.

! The program will compute amplifications at specified frequencies or at
! frequencies corresponding to the "breakpoints" in the velocity model.  

! The calculation of amplification at specified frequencies allows the program
! to generate automatically a stairstep model for 
! portions of a velocity model represented by a gradient  (program pwr2lyr
! can be used to generate a layered model for velocity vs depth described by a
! single power law).  This is very useful in preparing velocity models for use
! in layered-matrix site response calculations (e.g., C. Mueller's RATTLE; see
! F4Rattle).  The previous version of the program (for which frequencies were 
! determined by the travel time to each breakpoint in the velocity model) 
! required specification of velocities at many intermediate breakpoints.  I 
! found that a spreadsheet was useful to prepare the input file, giving depth 
! and velocity values at a number of depths spanned by linear velocity 
! gradients or to break up thick constant velocity layers into a number of 
! thinner layers.  This is no longer required.

! Q or 1/Q can be specified, and the cumulative kappa through the velocity
! model will be computed (starting from the top).  This value of kappa or a 
! specified kappa can be used to compute the diminution due to kappa.

! Density can be specified in three ways:
!
! 1) read from model file, with values of 0.0 replaced using a density--Vs relation
! 2) Computed from Vs using a built-in density--Vs relation (in site_amp, it was
!    possible to specify coefficients of this relation, but I have not allowed that
!    option here).
! 3) Reading in dens_default and dens_ref (the simplest case, allowing density
!    variations to have minimal or no impact).

! Here is a sample input file (column 1 of the input starts after the "*"
! character):
! Sample input file to site_amp; not for a specific application
!       Depth    SVel     Dens   1/Q
!           0   0.300      2.0   0.10
!       0.001   0.300      2.0   0.10  ! constant from 0.000 to 0.001
!       0.005   0.500      0.0   0.06  ! linear gradient from 0.001 to 0.005
!       0.020   0.800      0.0   0.05  ! linear gradient from 0.005 to 0.020
!       0.040   1.000      0.0   0.04  ! linear gradient from 0.020 to 0.040
!       0.040   1.500      0.0   0.04  ! step increase at 0.040
!       0.100   1.500      2.5   0.02  ! constant vel from 0.040 to 0.100
!       0.300   2.000      0.0   0.02  ! linear gradient from 0.100 to 0.300
!       0.300   2.500      0.0   0.02  ! step increase at 0.300
!       0.500   2.500      2.6   0.01  ! constant from 0.300 to 1.000,
!       1.000   2.500      2.6   0.001 !   but density and 1/Q differ

!And here is a sample control file:
!!Control file for site_amp_batch
!! Revision of program involving a change in the control file on this date:
!   03/15/22
!!Name of summary file:
!   site_amp_batch.sum
!!Are the velocities from a suspension log survey?
!!If yes, assume that the velocity value is at the midpoint of the layer
! N
!!Velocity for first layer (0 = use the first velocity of the input)
!! Use this for sensitivity studies of vavg for PS log models that do not
!! extend to the surface.   
!   100.0
!!Depth to bottom (d2b) or layer thickness (thk) (note: a value of 0.0 for
!!the first depth from the model file will signal that the model is a
!!piecewise continuous model)?
!   thk
!!Lines to skip to reach data (nskip):
!! nskip<0: use skipcmnt to skip a block of header lines starting with the 
!!          exclamation character ("!"), followed by skipping abs(nskip)-1 
!!          header lines.  This option is included so that the program can compute
!!          the amplifications for a group of model files that have a variable
!!          number of headers startng with "!" before a line with column headings.
!! nskip=0: skip no lines
!! nskip>0: skip the number of lines specified by nskip
!!
!! -1 ! the data are preceded by one or more lines starting with "!"
!! -2 ! the data are preceded by one or more lines starting with "!", followed 
!!      by a single line with column headers.
!   1 ! skip one line to reach the lines of model data
!!
!Use density in model file?
!!Lines to skip to reach data (<0, use skipcmnt; 0, skip none; > 0, specify)
!   1
!!Use density in model file?
! Y
!!dens_default (if layer density = 0.0, either because the densities from the 
!!  input file are to be used, but these are 0.0, or because the density is not
!!  to be obtained from the input file, then use this as the layer density unless it
!!  is 0.0, in which case obtain the density from my density--velocity
!!  subroutine):
!  0.0
!!Use Q from model file (note that Q < 1.0 is taken is 1/Q)?
! N
!! kappa for computation; this kappa is in addition to the total kappa accumulated from qinverse in each layer,
!! if Use Q from model file = Y (enter 0.0 for no kappa computation other than from qinverse):
! 0.0
!! **** NOTE **** as of 2021-08-29, the parameter below is no longer used.  But to avoid modifying the control file
!! I am retaining the entry here, as a placeholder.
! 1.0
!!Ncolumns to read, column numbers for depth or thickness (allows for the first column
!!being an index),velocity, density, and Q (note that if density and/or Q are not read
!!from the file, the column numbers for these variables will be ignored.  But dummy
!!values need to be included in the control file).
!! WARNING: Be careful to maintain consistency between whether density and/or Q
!! are read from input file and the entries below.  For example, suppose the input file
!! contains thickness, vs, density, and Q.  Then the following table illustrates the
!! proper input for Ncolumns, etc:
!** IMPORTANT **  Ncolumns to read should be consistent with the number of model columns actually present, although
! placeholders are needed so that there are 5 entries in the line below (see examples in the table below).
! Density from input  Q from input  Ncolumns, etc
!          N               N        2 1 2 2 2
!          Y               N        3 1 2 3 3
!          Y               Y        4 1 2 3 4
!          N               Y        4 1 2 3 4 (I'm not sure that this works)
!! If specified incorrectly, the program will probably work, but the velocity
!! model will be incorrect.  ALWAYS CHECK THE VELOCITY MODEL IN THE OUTPUT FILE!
!   3 1 2 3 3
!!Enter units of velocity (kps, mps) (depth units are assumed to be consistent
!!with the velocity units---km for kps and m for mps):
!   mps
!!vel_ref, dens_ref for amp calculation (dens_ref = 0.0 means that it will be obtained from
!! my relation between vel and dens; if dens_ref /= 0.0, then it will be used even if
!! "Use density from model file" is N):
!  3500  2.73
!!Angle of incidence at source level (in degrees):
! 0.0
!!Enter freqlow, freqhigh, nfreq for creation of CVL model and for computing amplifications.
!!  if nfreq  = 0: the CVL model will be based on the depths to the interfaces of the input model 
!!                      (even if the input model is made up of linear segments; if it is a CVL model, the model used for the amp calcs will be
!!                      the same as the input model).
!!     nfreq  < 0: the greatest depth for the CVL model will correspond to the bottom of the input model, with 
!!                      abs(nfreq) layers whose boundaries are determined by a range of frequencies from
!!                      the QWL frequency corresponding to the bottom of the input model (this replaces freqlow) to 
!!                      fhigh.
!!     nfreq  > 0: the CVL model will have nfreq layers whose boundaries are determined by the range of QWL depths corresponding to flow 
!!                      to fhigh. Note that it is possible for the deepest depth to be shallower or deeper than the bottom of the input model,
!!                      depending on the comparison of flow to the QWL frequency for the bottom of the input model.  If deeper, the model is
!!                      essentially extrapolated as a halfspace with velocity and density corresponding to the bottom of the input model.
! 0.01 50 400
!!Log-spaced frequencies (I suggest "Y" based on some sample runs, for which there were many oscillations in the amplifications at lower frequencies, due to
!! coarse layering at deeper depths)?
! Y
!!Enter freqlow, freqhigh, nfreq (if nfreq = 0, then the program will compute
!!amplifications at frequencies corresponding to the depths of interfaces, but 
!still need placeholders for freqlow and freqhigh):
! 0.01 50 400
!!Log-spaced frequencies?
! Y
!!Character string to append to end of each input file to create
!!the name of the output file (be sure to include the period if wanted):
! .test_batch_amps.asc
!!List of input files
!!Make sure that there are no blank entries!
!!The format of the input are consecutive columns containing the 
!!depth/thickess and velocity, after header lines starting with "!"
!thick.mdl
!hick2.mdl
!Stop
  
! NOTES:  

! --  I include a correction for cosine angle of incidence, BUT the travel 
!     times are computed assuming vertical incidence.


! Dates: 08/19/07 - Written by D.M. Boore, based on site_amp
!        09/10/07 - Write density coeffs not specified so have same number of
!                   headers as site_amp (this is required so that f4nrattle
!                   works).
!        10/04/10 - Added date to control file and allow for specification
!                   of reference density that is different than from my subroutine,
!                   even if the model densities are from the subroutine.  Change name
!                   of "dens_surf" to "dens_default"; reorder some of the control input
!        10/20/10 - Allow longer output filename
!        04/10/11 - Increase precision of some of output
!        07/25/12 - Add frequency below which halfspace affects the average velocity as
!                   the last column of output, for use in plotting the amplifications
!                   (they should only be plotted for frequencies above this frequency).
!        07/26/12 - Add units of velocity to output header line, and add columns at 
!                   end with model in the other units
!        08/04/12 - Increase resolution of velocity output (this is necessary if
!                   fine depth intervals are used with relatively large velocities in
!                   m/s).  The easiest way to do this without completely redoing the 
!                   output code is to change the size of cbuf to 11 characters and write
!                   the output variables using es11.5, although that is too much precision for
!                   most the variables.
!        08/13/12 - Use Day's (1996) criterion for the frequency below which halfspace affects 
!                   the average velocity as the last column of output, for use in plotting the 
!                   amplifications (they should only be plotted for frequencies above this frequency).  
!                   The new criterion is f = 1/(two-way travel time)= 0.5 Vs/Zmax 
!                   (rather than 0.25 Vs/Zmax, as before)..
!        08/23/12 - Go back to writing out the 1/4l frequency for the bottom of the model         
!        03/08/13 - Increased dimensions of cmnts2skip
!        08/13/14 - Fix bug that akkap4amp was not written to the output file
!        07/09/15 - Fix bug related to values of last two columns being constant for depths
!                   greater than the model depths
!        01/02/16 - Add slowness to output (in ms/m)
!        02/12/16 - Modernize some of the code
!        07/25/17 - Put building of the constant velocity model into a subroutine, and computed the 
!                   total effective inverse Q for the model.  This requires a change to the ctl file
!                   (to specify the power of the ratio of local to maximum slowness used in 
!                   apportioning the specified kappa_0 into each layer).
!        07/27/17 - Add qi_layer from apportioning k0_fixed into layers to output
!        07/28/17 - Want fixed number of headers, so disable printing of dens and Q from model lines
!                   for each layer (I've commented out the code)
!        07/29/17 - Modify qi model for input with gradients (the proper way is to constrain integral qi/vel, both 
!                   qi and vel having linear depth dependence; I am not doing that here (I need an analytic integral).
!        07/09/18 - Change the "lines to skip" input so that can deal with case of skipping a bunch of lines preceeded 
!                   by "!", but then skip header lines without "!" before them (usually only one line, containing
!                   column headers).
!        08/23/21 - Add avgdens2lyrbot to output
!        08/28/21 - Some formatting to enhance readability.
!        08/29/21 - For reasons discussed in the comments in the subroutine construct_constant_velocity_model (included in 
!                   site_amp_util_subs.for), I no longer apportion kappa to
!                   the output layers, and therefore I have deleted the 'qi_layr' and 'qik0fix' columns from the output.  Note
!                   that if the model has Q, it will be used to compute the kappa from the surface to the bottom of the model
!                   and the sum of this kappa and akappa_fixed will be used in the operator exp(-pi*kappa*f)
!        09/03/21 - Add tt in layer to output
!        09/15/21 - Add nentries_model, fix bug when ndepths > nout
!        09/18/21 - Replace "." with "NA" for missing entries
!        10/26/21 - Fix bug that program gave erroneous output for multiple model files.
!                   The error seems to be that nu_debug was closed after the first mdl file was processed, and writes to nu_debug
!                   for subsequent mdl files were written to the output file instead.  I fixed the bug by moving the close of nu_debug
!                   to after "END DO loop over files"
!        03/05/22 - Various revisions related to reinstating qi in the output model (for use in nrattle) for constant velocity layered models.
!        03/15/22 - Attempt to simplify code when possible.  For example, there is no reason to treat constant and linear density and also 
!                   constant and linear qinverse separately; just use the linear equations with slope=0 if constant.
!        03/17/22 - Add dens_int, as it is the best way of computing densavg and is useful in constructing the constant velocity model
!                 - Set the halfspace qi to 0.0
!        01/22/23 - Add SRImod as the last column in the output file
!        02/02/23 - Add SRImod from csv file (the previous value was from the function)
!        12/01/23 - Change name of include file with eta
!                 - Add an include file with fnorm corresponding to eta. NOTE: These files were created in C:\sri.underestimate_gradient_response\eta_revision,
!                   but I've copied them to C:\site_amp\programs because I could not get the program to read them from the folder in which they were created.
!                 - Use fnorm from include file, and add fnorm as the last column of the output (for comparison with the known eta vs fnorm plot)
!        02/11/24 - Major restructuring, putting the code after a piecewise continuous model has been developed into the 
!                   construct_constant_velocity_model subroutine (requiring a major change in that subroutine).  Doing this allows
!                   for extrapolating the input model, if desired, within the main program, and then calling construct_constant_velocity_model
!                   with that model.  I do need to compute and save fmin4OrigModel and fmin4model_final, as those frequencies will be used in constructing
!                   normalized frequencies for use in a later program that computes FR, SRI, and eta in one program.
!        02/22/24 - Change text before fmin so that f4nrattle finds fmin starting at t146 (this is to make f4nrattle consistent with previous versions of
!                   site_amp_batch output).
!        02/23/24 - Removed ndepthsAfterAnyExtrapolation and fmin4XtrapModel variable. It was there in case an extrapolation option of the original model was an option, but
!                   I decided against including that option.

      implicit none

      integer, parameter :: MLAYR = 10000
      integer, parameter :: MNFREQS =4000
      
      real dens_default,akappa_fixed,placeholder,dens_ref_in,vel_ref,  
     :   aoi_src, p,freqhigh,freqlow,slowness_layer,etaFromFunction,  
     :   EtaFromCSV, dd_dv, vel_high, dens_high, vel_low, dens_low, 
     :   dens_ref, fmin4OrigModel, freqhigh4model, 
     :   freqlow4model, fmin4model

      integer icol_q, icol_dens, icol_vel, icol_dh, ncol, nc_units_c, 
     :        nfreq_in, nc_ext, nu_debug, nc_f_mdl, nc_f_out, nu_out, 
     :        nu_mdl, nskip_more, nentries,  i, nlayers, idepth, 
     :        ilayer, ndepths, nout, 
     :        nfreq4model, ndx, j 

      integer nEtaCSV, nc_f_ctl,nu_ctl, nc_cmnts2skip, nc_version_in,
     :        nc_version_ctl, nc_f_sum, nu_sum,nc_buf, 
     :        nc_type_of_depth,  nskip, 
     :        ndepthsOriginalModel 

      real    pi,d2r, vel1, density, 
     :        EtaForSRImodFromFunction, EtaForSRImodFromCSV 

      real depth(MLAYR), vel(MLAYR), dens(MLAYR), qi(MLAYR),
     :     akappa(MLAYR), 
     :     d2b(0:MLAYR), 
     :     depth_in(MLAYR), vel_in(MLAYR), dens_in(MLAYR), 
     :     q_in(MLAYR)

      real thick_layer(MLAYR), vel_layer(MLAYR), dens_layer(MLAYR),
     :     qi_kappa_fixed(MLAYR), qi_layer(MLAYR), 
     :     d2lyrbot(0:MLAYR), tt2lyrbot(0:MLAYR), 
     :     avgdens2lyrbot(0:MLAYR), akappa2lyrbot(0:MLAYR), 
     :     avgvel2lyrbot(MLAYR), avgaoi2lyrbot(MLAYR), 
     :     freq_out(MNFREQS), 
     :     amp_out(MNFREQS), akappa_out(MNFREQS), 
     :     ampxkappa_out(MNFREQS), akappa4amp 

      real dum(10)
      

      real meanEtaFromFile(1200), alogfFromEtaCSV(1200)

      integer iout(10000), status

      character f_ctl*120, f_sum*120, f_mdl*120, f_out*120,
     :          constant_vel*1, dtbot*1, ext*100,
     :          buf*120, buf4*4,
     :          answer*1, tempbuf*8, 
     :          bffr*80, rho_col_exist*3, use_rho_col*3
      character col_d*10, col_v*10, col_r*10, col_qi*10, 
     :          col_f*10, col_a*10, col_k*10, col_ak*10,
     :          cbuf(28)*11, spcfy_freq*1, units_c*10,
     :          col_z_other_units*11, col_v_other_units*11
      character file4MeanEta*200
  
      character cmnts2skip(200)*100, type_of_depth*10

      character version_ctl*8, version_in*30

      logical susp_log, dens_from_model, q_from_model,
     :        f_mdl_exist, f_exist, specify_freqs,
     :        computeSRImodFromMeanEta
      logical specify_dens_coeff
      logical log_spaced_freqs 

!      include 'csv_data_statement.for'
      include 'DataStatement.EtaFromCSV.for'     ! meanEtaFromFile  
      include 'DataStatement.FnormFromCSV.for'   ! alogfFromEtaCSV
      nEtaCSV = 1200

!      dlogfNorm = 0.00291
!      alogfFromEtaCSV(1) = -0.80675221
!      do i = 2, nEtaCSV
!        alogfFromEtaCSV(i) = real(i-1)*dlogfNorm + alogfFromEtaCSV(1)
!      end do
      
      pi = 4.0 * atan(1.0)
      d2r = pi/180.0
 

!**********
!
!  GET MODEL-FILE INDEPENDENT INPUT
!
!**********

!Get name of control file and open the file:

      write(*,*)

      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :      '   Enter name of control file (cr=site_amp_batch.ctl; '//
     :         'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) == '    ') f_ctl='site_amp_batch.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

!Check version of control file
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      version_in = ' '
      read(nu_ctl,'(a)') version_in
      call trim_c(version_in,nc_version_in)
      
      version_ctl = ' '
      version_ctl = '07/09/18'
      call trim_c(version_ctl,nc_version_ctl)

      if (version_ctl(1:nc_version_ctl) /= 
     :    version_in(1:nc_version_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong version date; STOP!'
        write(*,'(a)') 
     :     ' The correct date is '//version_ctl(1:nc_version_ctl)
        close(nu_ctl)
        stop
      end if
      
 
!Get summary file name:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')
      write(nu_sum,'(9x,a, 3x,a, 1x,a, 1x,a, 4x,a, 1x,a)')
     :   'file', 'd2bot', 'thk_max', 'd_xtrap', 'v30', 'class'

!Are velocities from a suspension log?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         susp_log = .true.
      else
         susp_log = .false.
      end if

!Get velocity of first layer:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) vel1
      
!Depth to bottom (d2b) or thickness (thk)?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      type_of_depth = ' '
      read(nu_ctl, '(a)') type_of_depth
      call trim_c(type_of_depth, nc_type_of_depth)
      call upstr(type_of_depth)

!Lines to skip (<0 means use skipcmnt; 0 means skip none):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) nskip
!DEBUG
      write (*,'(1x,a,1x,i3)') ' nskip = ', nskip
!DEBUG

!Get density from the model file?:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         dens_from_model = .true.
      else
         dens_from_model = .false.
      end if
      
      specify_dens_coeff = .false.  ! The use of this logical variable 
                                    ! is a carryover from earlier versions;
                                    ! it is needed by the subroutine density() 

!Get dens_default (see comments in control file; set to 0.0 if want the density
! to be obtained from my density--velocity subroutine):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) dens_default 
      
!Get Q from the model file?:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         q_from_model = .true.
      else
         q_from_model = .false.
      end if

!Specify kappa:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) akappa_fixed

!Read placeholder (a legacy from a previous version:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) placeholder

!Ncolumns, Column numbers for depth/thickness, velocity, density, Q:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) ncol, icol_dh, icol_vel, icol_dens, icol_q

!Specify units of velocity (kps, mps):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      units_c = ' '
      read(nu_ctl,'(a)') units_c
      call trim_c(units_c, nc_units_c)
      call upstr(units_c)
      if (units_c(1:3) /= 'MPS' .and. units_c(1:3) /= 'KPS') then
        print *, ' ERROR: units_c(1:3) >= MPS or KPS; it equals '//
     :           units_c(1:3)//' STOP!!!'
        stop
      end if
      
      col_z_other_units = ' '
      col_v_other_units = ' '
      if (units_c(1:3) == 'MPS') then
!        col_z_other_units = '12345678901'
        col_z_other_units =  '      z(km)'
        col_v_other_units =  '    v(km/s)'
      else
        col_z_other_units =  '       z(m)'
        col_v_other_units =  '     v(m/s)'
      end if
      

      
!Get reference velocity and density (set to 0.0 if want the densities to be obtained
! from my density--velocity subroutine):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) vel_ref, dens_ref_in
      
!Enter angle-of-incidence at source level (in degrees): '
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) aoi_src
      p = sin(d2r*aoi_src)/vel_ref  ! ray parameter
      
!Enter freqlow, freqhigh, nfreq (if nfreq = 0, then the program will compute
!amplifications at frequencies corresponding to the depths of interfaces):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) freqlow, freqhigh, nfreq_in
      if (nfreq_in == 0) then
         specify_freqs = .false.
      else
         specify_freqs = .true.
      end if
      
!log-spaced frequencies?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         log_spaced_freqs = .true.
      else
         log_spaced_freqs = .false.
      end if
 
 
! Get extension for output file name:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      ext = ' '
      read(nu_ctl, '(a)') ext
      call trim_c(ext,nc_ext)
 
! Read and process files:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)

!DEBUG
      call get_lun(nu_debug)
      open(unit=nu_debug,file='debug.out',status='unknown')
!DEBUG
      
!**********
!
!  READ AND PROCESS VELOCITY MODELS
!
!**********


      loop over files: DO

        buf = ' '
        read(nu_ctl,'(a)',iostat=status) buf
        if(status /= 0) then
          EXIT loop over files
        end if
        call trim_c(buf, nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if(nc_buf == 0 .or. buf(1:1) == '!') then
          CYCLE loop over files
        end if
        if(buf4(1:4) == 'STOP') then
          EXIT loop over files
        end if
      
        f_mdl = ' '
        f_mdl = buf(1:nc_buf)
        call trim_c(f_mdl, nc_f_mdl)
        f_exist = .false.
        inquire(file=f_mdl(1:nc_f_mdl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') 
     :  ' ******* FILE '//f_mdl(1:nc_f_mdl)//' DOES NOT EXIST ******* '
          write(*,'(a)') ' Skipping to next file'
          CYCLE
        end if
 
!########################################## 
!Get name of output file and open the file:

        f_out = ' '
        f_out = f_mdl(1:nc_f_mdl)//ext(1:nc_ext)
        call trim_c(f_out, nc_f_out)
        
        call get_lun(nu_out)
        open(unit=nu_out, file=f_out(1:nc_f_out), status='unknown')

! Write name of model file:
        write(*,'(a)') ' Processing model file: '//f_mdl(1:nc_f_mdl)
        
        
        call get_lun(nu_mdl)
        open(nu_mdl,file=f_mdl(1:nc_f_mdl),status='unknown')
        
        if (nskip < 0) then
          call skipcmnt(nu_mdl, cmnts2skip, nc_cmnts2skip)
          nskip_more = abs(nskip) - 1
          call skip(nu_mdl, nskip_more)  
        else if (nskip > 0) then
          call skip(nu_mdl, nskip)
        end if
        
        
        nentries = 0 
        loop over entries: DO
        
          read(nu_mdl,*,iostat=status) (dum(i), i = 1, ncol)
	  
          if (status /= 0) then
            EXIT loop over entries
          end if
	  
          nentries = nentries + 1
      	  
          depth_in(nentries) = dum(icol_dh)
          vel_in(nentries) = dum(icol_vel)
          if(vel_in(nentries) == 0.0) then      ! check for the last layer
            nentries = nentries - 1      
          EXIT loop over entries
          end if
      
          if (dens_from_model) then
            dens_in(nentries) = dum(icol_dens)
!          write(nu_out, '(a, 1x,i5, a)')
!     :   ' For input model layer ', nentries, 
!     :             ' density is from the model file '
          else
            dens_in(nentries) = 0.0
!          write(nu_out, '(a, 1x,i5, a)')
!     :   ' For input model layer ', nentries, 
!     :             ' density is from the subroutine '
          end if
      
          if (q_from_model) then
            q_in(nentries) = dum(icol_q)
!          write(nu_out, '(a, 1x,i5, a)')
!     :   ' For input model layer ', nentries, 
!     :             ' 1/Q is from the model file '
          else
            q_in(nentries) = 0.0
!          write(nu_out, '(a, 1x,i5, a)')
!     :   ' For input model layer ', nentries, ' 1/Q = 0.0 '
          end if
    
        END DO loop over entries
        
        close(unit=nu_mdl)

        write(nu_out, '(2a, 1x,i5)') ' model file: '//f_mdl(1:nc_f_mdl),
     :                             ' nentries = ', nentries

        if (dens_from_model) then
          write(nu_out, '(a)')
     :   ' density in each layer is from the model file unless the '//
     :    'entry is 0.0 '
        else
          write(nu_out, '(a)')
     :   ' density in each layer is from the density subroutine '
        end if

        if (q_from_model) then
          write(nu_out, '(a)')
     :   ' 1/Q in each layer is from the model file '
        else
          write(nu_out, '(a)')
     :   ' 1/Q in each layer = 0.0 '
        end if

! Find out something about the type of velocity model:

        if (depth_in(1) == 0.0) then
          write(*,*)
          write(*, '(a)') 
     :  '   Assuming the model in file '//f_mdl(1:nc_f_mdl)//
     :  ' to be PIECEWISE CONTINUOUS;'//
     :    ' if not, stop program using ctl-brk, modify input file,'//
     :    ' and start again. '
          constant_vel = 'N'
        else
          constant_vel = 'Y'
        end if
 
! If constant velocity layers, convert into a velocity-depth function (with
! repeated depths at interfaces) so that it is in the same format as a 
! piecewise continuous function.

! **** NOTE: the depths here are for the model, not the depths corresponding to output frequencies
!      which can be much greater than the model depth ******

        IF (constant_vel == 'Y') THEN ! NOTE: There are nlayers of entries, even if the "depth" is d2b rather than thick
          nlayers = nentries
          idepth = 0
          d2b(0) = 0.0
          DO ilayer = 1, nlayers  
! first fill in depth-to-bottom array:
            if (type_of_depth(1:nc_type_of_depth) == 'D2B') then
              d2b(ilayer) = depth_in(ilayer)
            else if (type_of_depth(1:nc_type_of_depth) == 'THK') then
              d2b(ilayer) = d2b(ilayer-1) + depth_in(ilayer)
            else
              print *,' type_of_depth = '//
     :                type_of_depth(1:nc_type_of_depth)
              print *,' which is not a legal value; QUITTING'
              close(nu_out)
              close(nu_sum)
              stop
            end if
          
! now assign array values for the interfaces above and below each layer:
!   first above:
            idepth = idepth + 1
            depth(idepth) = d2b(ilayer-1)
            vel(idepth) = vel_in(ilayer)
            dens(idepth) = dens_in(ilayer)  
          
!  Fill in density from velocity-density relation if density = 0.0

            if (dens(idepth) == 0.0) then  
              if (dens_default == 0.0) then
                dens(idepth) = density(vel(idepth),
     :                     specify_dens_coeff,
     :                     dens_low, vel_low,
     :                     dens_high, vel_high, dd_dv, units_c)
              else
                dens(idepth) = dens_default
              end if
            end if
          
            if (q_from_model) then
              qi(idepth) = q_in(ilayer)
            else
              qi(idepth) = 0.0
            end if
          
            if (qi(idepth) > 1.0) qi(idepth) = 1.0/qi(idepth) ! cnvrt to 1/q

!   now below:
            idepth = idepth + 1
            depth(idepth) = d2b(ilayer)
            vel(idepth) = vel_in(ilayer)
            dens(idepth) = dens_in(ilayer)           
!  Fill in density from velocity-density relation if density = 0.0
            if (dens(idepth) == 0.0) then  
              if (dens_default == 0.0) then
                dens(idepth) = density(vel(idepth),
     :                     specify_dens_coeff,
     :                     dens_low, vel_low,
     :                     dens_high, vel_high, dd_dv, units_c)
              else
                dens(idepth) = dens_default
              end if
            end if
            if (q_from_model) then
              qi(idepth) = q_in(ilayer)
            else
              qi(idepth) = 0.0
            end if
            if (qi(idepth) > 1.0) qi(idepth) = 1.0/qi(idepth) ! cnvrt to 1/q
          END DO
          ndepths = idepth

        ELSE                 ! constant_vel /= 'Y'

          ndepths = nentries
          DO idepth = 1, ndepths
            depth(idepth) = depth_in(idepth)
            vel(idepth) = vel_in(idepth)
            dens(idepth) = dens_in(idepth)
!  Fill in density from velocity-density relation if density = 0.0
            if (dens(idepth) == 0.0) then  
              if (dens_default == 0.0) then
                dens(idepth) = density(vel(idepth),
     :                     specify_dens_coeff,
     :                     dens_low, vel_low,
     :                     dens_high, vel_high, dd_dv, units_c)
              else
                dens(idepth) = dens_default
              end if
            end if
            if (q_from_model) then
              qi(idepth) = q_in(idepth)
            else
              qi(idepth) = 0.0
            end if
            if (qi(idepth) > 1.0) qi(idepth) = 1.0/qi(idepth) ! cnvrt to 1/q
          END DO
        
        END IF

        if (dens_ref_in == 0.0) then
          dens_ref = density(vel_ref,
     :                     specify_dens_coeff,
     :                     dens_low, vel_low,
     :                     dens_high, vel_high, dd_dv, units_c)
        else
          dens_ref = dens_ref_in
        end if
         

        tempbuf = ' '
        tempbuf(1:8) = f_mdl(1:index(f_mdl,'.')-1)
        col_d = ' '
        col_d(9-len_trim(tempbuf):8) = tempbuf(1:len_trim(tempbuf))
        col_d(9:10) = '-d'
        col_v = ' '
        col_v(9-len_trim(tempbuf):8) = tempbuf(1:len_trim(tempbuf))
        col_v(9:10) = '-v'
        col_r = ' '
        col_r(9-len_trim(tempbuf):8) = tempbuf(1:len_trim(tempbuf))
        col_r(9:10) = '-r'
        col_qi = ' '
        col_qi(9-len_trim(tempbuf):8) = tempbuf(1:len_trim(tempbuf))
        col_qi(9:10) = 'qi'
        col_f = ' '
        col_f(9-len_trim(tempbuf):8) = tempbuf(1:len_trim(tempbuf))
        col_f(9:10) = '-f'
        col_a = ' '
        col_a(9-len_trim(tempbuf):8) = tempbuf(1:len_trim(tempbuf))
        col_a(9:10) = '-a'
        col_k = ' '
        col_k(9-len_trim(tempbuf):8) = tempbuf(1:len_trim(tempbuf))
        col_k(9:10) = '-k'
        col_ak = ' '
        col_ak(9-len_trim(tempbuf):8) = tempbuf(1:len_trim(tempbuf))
        col_ak(9:10) = 'ak'
        
! NOW THE MODELS ARE IN THE SAME FORMAT (CONTINUOUS WITH DEPTH, EVEN IF STEPS EXIST)
! AND WE CAN TREAT IT IN THE SAME WAY.

!**********
!
! CONSTRUCT THE EQUIVALENT CONSTANT-VELOCITY LAYERED MODEL:
!
!**********

       ndepthsOriginalModel = ndepths
       call construct_constant_velocity_model(
     :   ndepthsOriginalModel,   
     :   depth, vel, dens, qi, 
     :   vel_ref, dens_ref, aoi_src, p,
     :   akappa_fixed,
     :   freqlow, freqhigh, nfreq_in, 
     :   log_spaced_freqs, 
     :   d2lyrbot, thick_layer, vel_layer, dens_layer, 
     :   qi_kappa_fixed, qi_layer, 
     :   tt2lyrbot, avgvel2lyrbot, avgdens2lyrbot, avgaoi2lyrbot,
     :   akappa2lyrbot,
     :   freq_out, amp_out, akappa_out, ampxkappa_out, akappa4amp, nout,
     :   fmin4OrigModel, MLAYR, MNFREQS )
       

!**********
!
!-------------- OUTPUT ---------------------------------------------
!
!**********
 
! 2021-09-15: Replaced ndepths with ndepths - 1, since ndepths was increased to add the top of the halfspace,
! but it is not the proper measure of the number of depths in the model above the halfspace
! 2024-02-13: No longer replace ndepths with ndepths - 1.  The previous replacement was needed because
! the continuous model used in construct_constant_velocity_model needed an interface at the bottom of the input
! model, in order for the model to be extended into the halfspace.  The increased value of ndepths was passed
! out of the subroutine.  But now the increase of ndepths is in the subroutine and is not passed out (nout contains
! the number of layers in the final model)

        write(nu_out, '(a,2(1x,es10.3),a,f5.1)') 
     :  ' source vel & dens = ',  vel_ref, dens_ref,
     :  ' angle-of-incidence at source level = ', aoi_src
        write(nu_out, '(a, 1x,es10.3,
     :                a, 1x,es10.3,
     :                a, 1x,es10.3,
     :                a, 1x,es10.3)') 
     :   ' akappa_fixed = ', akappa_fixed, 
     :   ' akappa_from_model_Q = ', akappa_out(nout),
     :   ' sum_for_kappa_operator = ', 
     :               akappa_fixed + akappa_out(nout),
     :   ' akappa4amp = ', akappa4amp
 
      

        write(nu_out, '(t4,a, t14,i4, t20,a, t34,i4, 
     :                 t40,a, t146,es10.3  )') 
     :   'ndepths = ', ndepths, 'nout = ', nout,
     :   ' minimum frequency for the model '//
     :   '(the quarter wavelength frequency'//
     :   ' for the bottom of the input model) = ',
     :     fmin4OrigModel 

        write(nu_out,'(1x,a)') ' Velocity units: '//units_c(1:3)//
     :                         ' Slowness units: ms/m'

        write(nu_out, '(
     :     2x,a, 2x,a, 2x,a, 2x,a, 
     :     1x,a, 3x,a,  
     :     4x,a, 3x,a, 3x,a,
     :     4x,a,
     :     1x,a,
     :     5x,a,
     A     6x,a, 5x,a, 6x,a,
     :     2x,a, 2x,a, 2x,a, 2x,a,
     :     2x,a,
     :     1x,a, 1x,a,
     :     4x,a,
     :     1x,a, 1x,a,
     :     2x,a, 3x,a,
     :     5x,a)') 
     :     col_d, col_v, col_r, col_qi,
     :     'd2layer_bot', 'tt2lyrbot',
     :     'thck_lyr', 'vel_layer', 'dens_layr',
     :     'qi_layer',
     :     'slownss_lyr',
     :     'ttlayer', 
     A     'avgvel', 'avgdens', 'avgaoi',      
     :     col_f, col_a, col_k, col_ak, 
     :     'fmin4model',
     :     col_z_other_units, col_v_other_units,
     :     'slowness',
     :     'etaFromFunc', 'SRImodFrmFn',
     :     'etaFromCSV', 'SRImodCSV',
     :     'f/fbot'

! col_d, ... col_ak have a length of 10 characters     
! col_z_other_units, col_v_other_units have a length of 11 characters     
      
        ndx = max0( ndepths, nout)
!DEBUG
        write(nu_debug,*)
        write(nu_debug,'(a,1x,i5)') ' ndx = ', ndx
        write(nu_debug,*)
!DEBUG
! Print when have entries in each column
        DO i = 1, ndx
        
    ! Initialize cbuf        
          DO j = 1, 28
            cbuf(j) = '         NA'
!            cbuf(j) = '          .'
          END DO
          
          IF (i <= ndepths) THEN
!             fill entries 1 -- 4 and 21 -- 23 of cbuf with values
            write(cbuf(1), '(es11.5)') depth(i)
            write(cbuf(2), '(es11.5)') vel(i)
            write(cbuf(3), '(es11.5)') dens(i)
            write(cbuf(4), '(es11.5)') qi(i)
            IF (units_c(1:3) == 'MPS') THEN
              write(cbuf(21), '(es11.5)') 
     :          depth(i)/1000.0
              write(cbuf(22), '(es11.5)') 
     :          vel(i)/1000.0
              write(cbuf(23), '(es11.5)') 
     :          1000.0/vel(i)
            ELSE
              write(cbuf(21), '(es11.5)') 
     :          1000.0*depth(i) 
              write(cbuf(22), '(es11.5)') 
     :          1000.0*vel(i) 
              write(cbuf(23), '(es11.5)') 
     :          1.0/vel(i) 
            END IF
          
          END IF


          IF (i <= nout) THEN
!              fill entries 5 -- 20 of cbuf with values
            write(cbuf(5), '(es11.5)') d2lyrbot(i) 
            write(cbuf(6), '(es11.5)') tt2lyrbot(i)
            write(cbuf(7), '(es11.5)') thick_layer(i)
            write(cbuf(8), '(es11.5)') vel_layer(i)
            write(cbuf(9), '(es11.5)') dens_layer(i)
            write(cbuf(10), '(es11.5)') qi_layer(i) 
            if (units_c(1:3) == 'MPS') then
              slowness_layer = 1000.0/vel_layer(i)
            else
              slowness_layer = 1.0/vel_layer(i)
            end if
            write(cbuf(11), '(es11.5)') slowness_layer
            write(cbuf(12), '(es11.5)') tt2lyrbot(i) - tt2lyrbot(i-1)
!            write(cbuf(12), '(es11.5)') qi_kappa_fixed(i) 
            write(cbuf(13), '(es11.5)') avgvel2lyrbot(i) 
            write(cbuf(14), '(es11.5)') avgdens2lyrbot(i) 
            write(cbuf(15), '(es11.5)') avgaoi2lyrbot(i)
            write(cbuf(16), '(es11.5)') freq_out(i)
            write(cbuf(17), '(es11.5)') amp_out(i)
            write(cbuf(18), '(es11.5)') 
     :            akappa_out(nout)+akappa_fixed
            write(cbuf(19), '(es11.5)') ampxkappa_out(i)
            write(cbuf(20), '(es11.5)') fmin4OrigModel
            etaFromFunction = 
     :         EtaForSRImodFromFunction(freq_out(i), fmin4OrigModel)
            write(cbuf(24), '(es11.5)') etaFromFunction
            write(cbuf(25), '(es11.5)') 
     :         amp_out(i)**(2.0*etaFromFunction)
            EtaFromCSV = 
     :         EtaForSRImodFromCSV(freq_out(i), fmin4OrigModel, 
     :            alogfFromEtaCSV, meanEtaFromFile,nEtaCSV)
            write(cbuf(26), '(es11.5)') EtaFromCSV
            write(cbuf(27), '(es11.5)') 
     :         amp_out(i)**(2.0*EtaFromCSV)
            write(cbuf(28), '(es11.5)') 
     :         freq_out(i)/fmin4OrigModel
 
          END IF

!      function EtaForSRImodFromCSV(f, f2b, alogfFromEtaCSV, 
!     :                             meanEtaFromEtaCSV,nEtaCSV)

          write(nu_out, '(28(1x,a11) )') 
     :      (cbuf(j),j=1,28)
!     :      (cbuf(j),j=1,11), 
!     :      (cbuf(j),j=13,23) 
        END DO                               ! loop over depths or nfreqs

! ---------------------------------------------------------------

        close(unit=nu_out)
     
        write(*, '(a)') ' FINISHED --- Loop back for another model'
        write(*,*)
      END DO loop over files
 
      close(nu_sum)
      close(nu_ctl)
 
!DEBUG
        close(unit=nu_debug)
!DEBUG


      END


      include 'site_amp_util_subs.for'
!      include 'construct_constant_velocity_model.for'     